from abc import ABC, abstractmethod

from django.conf import settings
from django.core.exceptions import ImproperlyConfigured
from django.urls import reverse


class BaseProcessor(ABC):
    production_url = None  #: Base URL of production environment.
    sandbox_url = None  #: Base URL of sandbox environment.
    display_name = None  #: The name of the provider for the ``choices``.
    accepted_currencies = None  #: List of accepted currency codes (ISO 4217).
    logo_url = None  #: Logo URL - can be used in templates.
    slug = None  #: For friendly urls
    method = "GET"  #: Method of operation. One of GET, POST or REST.
    template_name = None  #: Template used in the POST method flow.

    def __init__(self, payment):
        self.payment = payment
        self.path = payment.backend
        self.context = {}  # can be used by Payment's customized methods.
        if self.slug is None:
            self.slug = self.path
        self.config = getattr(settings, "GETPAID_BACKEND_SETTINGS", {}).get(
            self.path, {}
        )
        self.optional_config = getattr(settings, "GETPAID", {})

    @classmethod
    def class_id(cls):
        return cls.__module__

    def get_setting(self, name, default=None):
        return self.config.get(name, default)

    @classmethod
    def get_display_name(cls) -> str:
        return cls.display_name

    @classmethod
    def get_accepted_currencies(cls) -> list:
        return cls.accepted_currencies

    @classmethod
    def get_logo_url(cls) -> str:
        return cls.logo_url

    def get_paywall_method(self) -> str:
        return self.method

    def get_form(self, post_data):
        """
        Only used if the payment processor requires POST requests.
        Generates a form only containing hidden input fields.
        """
        from . import forms

        return forms.PaymentHiddenInputsPostForm(items=post_data)

    def get_paywall_baseurl(self):
        if settings.DEBUG:
            return self.sandbox_url
        return self.production_url

    def get_paywall_url(self, params=None):
        """
        Provide URL to paywall. If method uses optional ``params`` argument,
        the resulting URL can be i.e. extracted from them (usually during REST
        flow) or constructed using them (usually during GET flow). Default
        implementation returns production or sandbox url based on DEBUG setting.
        """
        return self.get_paywall_baseurl()

    def get_template_names(self, view=None) -> list:
        template_name = self.get_setting("POST_TEMPLATE")
        if template_name is None:
            template_name = self.optional_config.get("POST_TEMPLATE")
        if template_name is None:
            template_name = self.template_name
        if template_name is None and hasattr(view, "get_template_names"):
            return view.get_template_names()
        if template_name is None:
            raise ImproperlyConfigured("Couldn't determine template name!")
        return [template_name]

    @abstractmethod
    def get_paywall_params(self, request) -> dict:
        """
        Gather all the data required by the broker.

        :param request:
        :return: Dict of all params accepted by paywall API.
        """
        raise NotImplemented

    def prepare_paywall_headers(self, obj: dict = None) -> dict:
        """
        Prepares HEADERS dict for REST or POST methods. This is where you will
        probably calculate the signature of ``obj``.

        :param dict obj: Serialized payment object that you can use to calculate signature.
        :return: Dictionary of headers that must be attached to a request to paywall.
        """
        raise NotImplemented

    def handle_paywall_response(self, response) -> dict:
        """
        Analyze direct response from broker, update payment status if applicable,
        and return dict with extracted and normalized data.

        :param response: is expected to be :py:mod:`request.response` object.
        """
        raise NotImplemented

    def handle_paywall_callback(self, request, *args, **kwargs):
        """
        This method handles the callback from payment broker for the purpose
        of asynchronously updating the payment status in our system.

        :return: HttpResponse instance
        """
        raise NotImplementedError

    def fetch_payment_status(
        self,
    ) -> dict:  # TODO use interface annotation to specify the dict layout
        """
        Logic for checking payment status with broker.

        Should return dict with either "amount" or "status" keys.
        If "status" key is used, it should be one of getpaid.models.PAYMENT_STATUS_CHOICES
        If both keys are present, "status" takes precedence.
        """
        raise NotImplementedError

    def lock(self, amount=None):
        """
        Lock given amount for future charge.
        Returns amount locked amount.
        """
        raise NotImplemented

    def charge_locked(self, amount=None):
        """
        Check if payment can be locked and call processor's method.
        This method is used eg. in flows that pre-authorize payment during
        order placement and charge money upon shipping.
        Returns charged amount.
        """
        raise NotImplemented

    def release(self):
        """
        Release locked payment. This can happen if pre-authorized payment cannot
        be fullfilled (eg. the ordered product is no longer available for some reason).
        Returns released amount.
        """
        raise NotImplemented

    def refund(self, amount):
        """
        Refunds the given amount.

        Returns the amount that was actually refunded.
        """
        raise NotImplemented



from django.dispatch import Signal

payment_status_changed = Signal(providing_args=["instance", "old_status", "new_status"])
payment_status_changed.__doc__ = """Sent when Payment status changes."""

payment_fraud_changed = Signal(
    providing_args=["instance", "old_status", "new_status", "message"]
)
payment_fraud_changed.__doc__ = """Sent when Payment fraud status changes."""



# Generated by Django 3.0.4 on 2020-04-05 01:52

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("getpaid", "0002_auto_20200403_1554"),
    ]

    operations = [
        migrations.AlterField(
            model_name="payment",
            name="amount_locked",
            field=models.DecimalField(
                decimal_places=2,
                default=0,
                help_text="Amount locked with this payment, ready to charge.",
                max_digits=20,
                verbose_name="amount paid",
            ),
        ),
        migrations.AlterField(
            model_name="payment",
            name="amount_paid",
            field=models.DecimalField(
                decimal_places=2,
                default=0,
                help_text="Amount actually paid.",
                max_digits=20,
                verbose_name="amount paid",
            ),
        ),
        migrations.AlterField(
            model_name="payment",
            name="amount_required",
            field=models.DecimalField(
                decimal_places=2,
                help_text="Amount required to fulfill the payment; in selected currency, normal notation",
                max_digits=20,
                verbose_name="amount required",
            ),
        ),
    ]



import json

from django.http import HttpResponse
from django.urls import reverse

from getpaid.processor import BaseProcessor


class PaymentProcessor(BaseProcessor):
    slug = "dummy"
    display_name = "Dummy"
    accepted_currencies = ["PLN", "EUR"]
    method = "POST"
    template_name = "getpaid_dummy_backend/payment_post_form.html"

    def handle_paywall_callback(self, request, *args, **kwargs):
        payload = json.loads(request.body)
        if payload["status"] == "OK":
            self.payment.on_success()
        else:
            self.payment.on_failure()

        return HttpResponse("OK")

    def get_paywall_params(self, request) -> dict:
        extra_args = {}
        if self.get_setting("confirmation_method", "push").lower() == "push":
            extra_args["callback"] = request.build_absolute_uri(
                reverse(
                    f"getpaid:{self.slug}:callback", kwargs={"pk": self.payment.pk}
                ),
            )

        return dict(
            payment=self.payment.pk,
            value=self.payment.amount_required,
            currency=self.payment.currency,
            description=self.payment.description,
            success_url=request.build_absolute_uri(
                reverse("getpaid:payment-success", kwargs={"pk": self.payment.pk})
            ),
            failure_url=request.build_absolute_uri(
                reverse("getpaid:payment-failure", kwargs={"pk": self.payment.pk})
            ),
            **extra_args,
        )

    def get_paywall_url(self, *args, **kwargs):
        return self.get_setting("gateway")



import uuid
from importlib import import_module

import pendulum
import swapper
from django.conf import settings
from django.db import models
from django.shortcuts import resolve_url
from django.utils.translation import ugettext_lazy as _

from . import FraudStatus, PaymentStatus, signals
from .registry import registry


class AbstractOrder(models.Model):
    class Meta:
        abstract = True

    def get_return_url(self, *args, success=None, **kwargs):
        """
        Method used to determine the final url the client should see after
        returning from gateway. Client will be redirected to this url after
        backend handled the original callback (i.e. updated payment status)
        and only if SUCCESS_URL or FAILURE_URL settings are NOT set.
        By default it returns the result of `get_absolute_url`
        """
        return self.get_absolute_url()

    def get_absolute_url(self):
        """
        Standard method recommended in Django docs. It should return
        the URL to see details of particular Order.
        """
        raise NotImplementedError

    def is_ready_for_payment(self):
        """Most of the validation is made in PaymentMethodForm using but if you
        need any extra validation. For example you most probably want to disable
        making another payment for order that is already paid."""
        return True

    def get_items(self):
        """
        There are backends that require some sort of item list to be attached
        to the payment. But it's up to you if the list is real or contains only
        one item called "Payment for stuff in {myshop}" ;)

        :return: List of {"name": str, "quantity": Decimal, "unit_price": Decimal} dicts.
        """
        return [
            {
                "name": self.get_description(),
                "quantity": 1,
                "unit_price": self.get_total_amount(),
            }
        ]

    def get_total_amount(self):
        """
        This method must return the total value of the Order.

        :return: Decimal object
        """
        raise NotImplementedError

    def get_user_info(self) -> dict:
        """
        This method should return dict with necessary user info.
        For most backends email should be sufficient.
        Expected field names: `email`, `first_name`, `last_name`, `phone`
        """
        raise NotImplementedError

    def get_description(self) -> str:
        """
        :return: Description of the Order. Should return the value of appropriate field.
        """
        raise NotImplementedError


class AbstractPayment(models.Model):
    id = models.UUIDField(primary_key=True, default=uuid.uuid4, editable=False)
    order = models.ForeignKey(
        swapper.get_model_name("getpaid", "Order"),
        verbose_name=_("order"),
        on_delete=models.CASCADE,
        related_name="payments",
    )
    amount_required = models.DecimalField(
        _("amount required"),
        decimal_places=2,
        max_digits=20,
        help_text=_(
            "Amount required to fulfill the payment; in selected currency, normal notation"
        ),
    )
    currency = models.CharField(_("currency"), max_length=3)
    status = models.CharField(
        _("status"),
        max_length=20,
        choices=PaymentStatus.CHOICES,
        default=PaymentStatus.NEW,
        db_index=True,
    )
    backend = models.CharField(_("backend"), max_length=100, db_index=True)
    created_on = models.DateTimeField(_("created on"), auto_now_add=True, db_index=True)
    last_payment_on = models.DateTimeField(
        _("paid on"), blank=True, null=True, default=None, db_index=True
    )
    amount_locked = models.DecimalField(
        _("amount paid"),
        decimal_places=2,
        max_digits=20,
        default=0,
        help_text=_("Amount locked with this payment, ready to charge."),
    )
    amount_paid = models.DecimalField(
        _("amount paid"),
        decimal_places=2,
        max_digits=20,
        default=0,
        help_text=_("Amount actually paid."),
    )
    refunded_on = models.DateTimeField(
        _("refunded on"), blank=True, null=True, default=None, db_index=True
    )
    amount_refunded = models.DecimalField(
        _("amount refunded"), decimal_places=4, max_digits=20, default=0
    )
    external_id = models.CharField(
        _("external id"), max_length=64, blank=True, db_index=True, default=""
    )
    description = models.CharField(
        _("description"), max_length=128, blank=True, default=""
    )
    fraud_status = models.CharField(
        _("fraud status"),
        max_length=20,
        choices=FraudStatus.CHOICES,
        default=FraudStatus.UNKNOWN,
        db_index=True,
    )
    fraud_message = models.TextField(_("fraud message"), blank=True)
    _processor = None

    class Meta:
        abstract = True
        ordering = ["-created_on"]
        verbose_name = _("Payment")
        verbose_name_plural = _("Payments")

    def __str__(self):
        return "Payment #{self.id}".format(self=self)

    @property
    def processor(self):
        if self._processor is None:
            self._processor = self.get_processor()
        return self._processor

    def get_items(self):
        """
        Some backends require the list of items to be added to Payment.

        This method relays the call to Order. It is here simply because
        you can change the Order's fieldname when customizing Payment model.
        In that case you need to overwrite this method so that it properly
        returns a list.
        """
        # TODO: type/interface annotation
        return self.order.get_items()

    def get_processor(self):
        """
        Returns the processor instance for the backend that
        was chosen for this Payment. By default it takes it from global
        backend registry and tries to import it when it's not there.
        You most probably don't want to mess with this.
        """
        if self.backend in registry:
            processor = registry[self.backend]
        else:
            # last resort if backend has been removed from INSTALLED_APPS
            module = import_module(self.backend)
            processor = getattr(module, "PaymentProcessor")
        return processor(self)

    def change_status(self, new_status):
        """
        Used for changing the status of the Payment.
        You should always change payment status via this method.
        Otherwise the signal will not be emitted.
        """
        if self.status != new_status:
            # do anything only when status is really changed
            old_status = self.status
            self.status = new_status
            self.save()
            signals.payment_status_changed.send(
                sender=self.__class__,
                instance=self,
                old_status=old_status,
                new_status=new_status,
            )

    def change_fraud_status(self, new_status, message=""):
        """
        Used for changing fraud status of the Payment.
        You should always change payment fraud status via this method.
        Otherwise the signal will not be emitted.
        """
        if self.fraud_status != new_status:
            # do anything only when status is really changed
            old_status = self.fraud_status
            self.fraud_status = new_status
            self.fraud_message = message
            self.save()
            signals.payment_fraud_changed.send(
                sender=self.__class__,
                instance=self,
                old_status=old_status,
                new_status=new_status,
                message=message,
            )

    @property
    def fully_paid(self):
        return self.amount_paid >= self.amount_required

    def on_success(self, amount=None):
        """
        Called when payment receives successful balance income. It defaults to
        complete payment, but can optionally accept received amount as a parameter
        to handle partial payments.

        Returns boolean value whether payment was fully paid.
        """
        if getattr(settings, "USE_TZ", False):
            timezone = getattr(settings, "TIME_ZONE", "local")
            self.last_payment_on = pendulum.now(timezone)
        else:
            self.last_payment_on = pendulum.now("UTC")

        if amount:
            self.amount_paid = amount
        else:
            self.amount_paid = self.amount_required

        if self.fully_paid:
            self.change_status(PaymentStatus.PAID)
        else:
            self.change_status(PaymentStatus.PARTIAL)

        return self.fully_paid

    def on_failure(self):
        """
        Called when payment has failed. By default changes status to 'failed'
        """
        self.change_status(PaymentStatus.FAILED)

    def get_paywall_method(self) -> str:
        """
        Interfaces processor's ``get_paywall_method``.

        Returns the method to be used to complete the payment - 'POST', 'GET', or 'REST.
        """
        return self.processor.get_paywall_method()

    def get_form(self, *args, **kwargs):
        """
        Interfaces processor's ``get_form``.

        Returns a Form to be used on intermediate page if the method returned by
        ``get_redirect_method`` is 'POST'.
        """
        return self.processor.get_form(*args, **kwargs)

    def get_paywall_url(self, params=None) -> str:
        """
        Interfaces processor's ``get_paywall_url``.

        Returns URL where the user will be redirected to complete the payment.

        Takes optional ``params`` which can help with constructing the url.
        """
        return self.processor.get_paywall_url(params)

    def get_template_names(self, view=None):
        """
        Interfaces processor's ``get_template_names``.

        Used to get templates for intermediate page when ``get_redirect_method``
        returns 'POST'.
        """
        return self.processor.get_template_names(view=view)

    def get_paywall_params(self, request) -> dict:
        """
        Interfaces processor's ``get_paywall_params``.

        Returns a dictionary containing all the data required by
        backend to process the payment in appropriate format.
        The data is extracted from Payment and Order.
        """
        return self.processor.get_paywall_params(request)

    def prepare_paywall_headers(self, obj: dict = None) -> dict:
        """
        Interfaces processor's ``prepare_paywall_headers``.

        Prepares headers for REST request to paywall.
        """
        return self.processor.prepare_paywall_headers(obj)

    def handle_paywall_response(self, response) -> dict:
        """
        Interfaces processor's ``handle_paywall_response``.

        Validates and dictifies any direct response from paywall.
        """
        return self.processor.handle_paywall_response(response)

    def handle_paywall_callback(self, request, *args, **kwargs):
        """
        Interfaces processor's ``handle_paywall_callback``.

        Called when 'PUSH' flow is used for a backend. In this scenario paywall
        will send a request to our server with information about the state of
        Payment. Broker can send several such requests during Payment's lifetime.
        Backend should analyze this request and return appropriate response that
        can be understood by paywall.

        :param request: Request sent by paywall.

        :return: HttpResponse instance
        """
        return self.processor.handle_paywall_callback(request, *args, **kwargs)

    def fetch_status(self):
        """
        Interfaces processor's ``fetch_payment_status``.

        Used during 'PULL' flow. Fetches status from paywall and translates it to
        a value from ``PAYMENT_STATUS_CHOICES``.
        """
        return self.processor.fetch_payment_status()

    def fetch_and_update_status(self):
        """
        Used during 'PULL' flow to automatically fetch and update
        Payment's status.
        """
        remote_status = self.fetch_status()
        status = remote_status.get("status", None)
        amount = remote_status.get("amount", None)
        if (
            status is not None and status in [PaymentStatus.PAID, PaymentStatus.PARTIAL]
        ) or amount is not None:
            self.on_success(amount)
        elif status == PaymentStatus.FAILED:
            self.on_failure()
        elif status is not None:
            self.change_status(status)
        return status

    def lock(self, amount=None):
        """
        Used to lock payment for future charge.
        Returns locked amount.
        """
        if amount is None:
            amount = self.amount_required
        amount_locked = self.processor.lock(amount)
        if amount_locked:
            self.amount_locked = amount_locked
            self.change_status(PaymentStatus.ACCEPTED)
        else:
            self.on_failure()
        return amount_locked

    def charge_locked(self, amount=None):
        """
        Charges the locked payment.
        This method is used eg. in flows that pre-authorize payment during
        order placement and charge money just before shipping.
        """
        if self.status != PaymentStatus.ACCEPTED:
            raise ValueError("Only accepted payments can be charged.")
        if amount is None:
            amount = self.amount_locked
        if amount > self.amount_locked:
            raise ValueError("Cannot charge more than is locked.")
        amount_charged = self.processor.charge_locked(amount)
        if amount_charged:
            self.amount_locked -= amount_charged
            self.on_success(amount_charged)
        return amount_charged

    def release(self):
        """
        Release locked payment. This can happen if pre-authorized payment cannot
        be fulfilled (eg. the ordered product is no longer available for some reason).
        """
        if self.status != PaymentStatus.ACCEPTED:
            raise ValueError("Only accepted (locked) payments can be released.")

        released_amount = self.processor.release()
        self.amount_locked -= released_amount
        self.change_status(PaymentStatus.REFUNDED)
        return released_amount

    def refund(self, amount=None):
        """
        Interfaces processor's ``refund``.

        :param: amount - optional refund amount - if not given, refunds paid value.
        :returns: the result of processor's ``refund`` method that should return the amount refunded.

        """
        if self.status not in [PaymentStatus.PAID, PaymentStatus.PARTIAL]:
            raise ValueError("Only paid paymets can be refunded.")
        if amount is None:
            amount = self.amount_locked
        if amount:
            if amount > self.amount_locked:
                raise ValueError("Cannot refund more than what was paid.")
            self.amount_refunded = self.processor.refund(amount)
            self.amount_locked -= self.amount_refunded
            self.refunded_on = pendulum.now()
            self.save()
            if self.amount_locked == 0:
                self.change_status(PaymentStatus.REFUNDED)
        return self.amount_refunded

    def get_return_redirect_url(self, request, success):
        fallback_settings = getattr(settings, "GETPAID", {})

        if success:
            url = self.processor.get_setting(
                "SUCCESS_URL", getattr(fallback_settings, "SUCCESS_URL", None)
            )
        else:
            url = self.processor.get_setting(
                "FAILURE_URL", getattr(fallback_settings, "FAILURE_URL", None)
            )

        if url is not None:
            # we may want to return to Order summary or smth
            kwargs = self.get_return_redirect_kwargs(request, success)
            return resolve_url(url, **kwargs)
        return resolve_url(self.order.get_return_url(self, success=success))

    def get_return_redirect_kwargs(self, request, success):
        return {"pk": self.order_id}


class Payment(AbstractPayment):
    class Meta(AbstractPayment.Meta):
        swappable = swapper.swappable_setting("getpaid", "Payment")



import swapper
from django.shortcuts import get_object_or_404
from django.views import View


class CallbackView(View):
    """
    Each plugin can define its own callbacks and other views.
    """

    def post(self, request, pk, *args, **kwargs):
        Payment = swapper.load_model("getpaid", "Payment")
        payment = get_object_or_404(Payment, pk=pk)
        return payment.handle_paywall_callback(request, *args, **kwargs)


callback = CallbackView.as_view()



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['getpaid', 'getpaid.backends', 'getpaid.backends.dummy', 'getpaid.migrations']

package_data = \
{'': ['*'], 'getpaid.backends.dummy': ['templates/getpaid_dummy_backend/*']}

install_requires = \
['django-model-utils>=4.0.0,<5.0.0',
 'pendulum>=2.0.5,<3.0.0',
 'requests>=2.23.0,<3.0.0',
 'sphinx-rtd-theme>=0.4.3,<0.5.0',
 'swapper>=1.1.2,<2.0.0']

setup_kwargs = {
    'name': 'django-getpaid',
    'version': '2.0.0rc6',
    'description': 'Multi-broker payment processor for Django.',
    'long_description': '=============================\nWelcome to django-getpaid\n=============================\n\n.. image:: https://img.shields.io/pypi/v/django-getpaid.svg\n    :target: https://pypi.org/project/django-getpaid/\n    :alt: Latest PyPI version\n.. image:: https://img.shields.io/travis/sunscrapers/django-getpaid.svg\n    :target: https://travis-ci.org/sunscrapers/django-getpaid\n.. image:: https://img.shields.io/coveralls/github/cypreess/django-getpaid.svg\n    :target: https://coveralls.io/github/django-getpaid/django-getpaid?branch=master\n.. image:: https://img.shields.io/pypi/wheel/django-getpaid.svg\n    :target: https://pypi.org/project/django-getpaid/\n.. image:: https://img.shields.io/pypi/l/django-getpaid.svg\n    :target: https://pypi.org/project/django-getpaid/\n\n\ndjango-getpaid is a multi-broker payment processor for Django\n\nDocumentation\n=============\n\nThe full documentation is at https://django-getpaid.readthedocs.io.\n\nFeatures\n========\n\n* support for multiple payment brokers at the same time\n* clean but flexible architecture\n* support for asynchronous status updates - both push and pull\n* support for modern REST-based broker APIs\n* support for multiple currencies (but one per payment)\n* easy customization with provided base abstract models and swappable mechanic (same as with Django\'s User model)\n\n\nQuickstart\n==========\n\nInstall django-getpaid and at least one payment backend:\n\n.. code-block:: console\n\n    pip install django-getpaid\n    pip install django-getpaid-paynow\n\nAdd them to your ``INSTALLED_APPS``:\n\n.. code-block:: python\n\n    INSTALLED_APPS = [\n        ...\n        \'getpaid\',\n        \'getpaid_paynow\',  # one of plugins\n        ...\n    ]\n\nAdd getpaid to URL patterns:\n\n.. code-block:: python\n\n    urlpatterns = [\n        ...\n        path(\'payments/\', include(\'getpaid.urls\')),\n        ...\n    ]\n\nUse ``getpaid.models.AbstractOrder`` as parent class of your Order model and define minimal set of methods:\n\n.. code-block:: python\n\n    from getpaid.models import AbstractOrder\n\n    class MyCustomOrder(AbstractOrder):\n        amount = models.DecimalField(decimal_places=2, max_digits=8)\n        description = models.CharField(max_length=128)\n        buyer = models.ForeignKey(settings.AUTH_USER_MODEL, on_delete=models.CASCADE)\n\n        def get_absolute_url(self):\n            return reverse(\'order-detail\', kwargs={"pk": self.pk})\n\n        def get_total_amount(self):\n            return self.amount\n\n        def get_user_info(self):\n            return {"email": self.buyer.email}\n\n        def get_description(self):\n            return self.description\n\n\nInform getpaid of your Order model in ``settings.py`` and provide settings for payment backends:\n\n.. code-block:: python\n\n    GETPAID_ORDER_MODEL = \'yourapp.MyCustomOrder\'\n\n    GETPAID_BACKEND_SETTINGS = {\n        \'getpaid_paynow\': {   # dotted import path of the plugin\n            # refer to backend docs for its real settings\n            "api_key": "9bcdead5-b194-4eb5-a1d5-c1654572e624",\n            "signature_key": "54d22fdb-2a8b-4711-a2e9-0e69a2a91189",\n        },\n    }\n\nAnd... provide some business logic ;)\n\nYour pre-payment view should use ``getpaid.forms.PaymentMethodForm`` `bound <https://docs.djangoproject.com/en/3.0/ref/forms/api/#ref-forms-api-bound-unbound>`_\nwith payment data. During binding the form will generate a list of plugins\n(payment methods) supporting your currency and hide rest of the fields.\nThen this form should be POSTed to ``{% url \'getpaid:create-payment\' %}`` to create\nnew payment. You should be automatically redirected to paywall. After payment\nyou should by default return to order-detail page but this behavior can be\nchanged by plugin\'s config.\n\nRunning Tests\n=============\n\nDoes the code actually work?\n\n.. code-block:: console\n\n    poetry install\n    poetry run tox\n\n\nCredits\n=======\n\nCreated by `Krzysztof Dorosz <https://github.com/cypreess>`_.\nRedesigned and rewritten by `Dominik Kozaczko <https://github.com/dekoza>`_.\n\nProudly sponsored by `SUNSCRAPERS <http://sunscrapers.com/>`_\n\nTools used in rendering this package:\n\n*  Cookiecutter_\n*  `cookiecutter-djangopackage`_\n\n.. _Cookiecutter: https://github.com/audreyr/cookiecutter\n.. _`cookiecutter-djangopackage`: https://github.com/pydanny/cookiecutter-djangopackage\n\n\nDisclaimer\n==========\n\nThis project has nothing in common with `getpaid <http://code.google.com/p/getpaid/>`_ plone project.\n',
    'author': 'Dominik Kozaczko',
    'author_email': 'dominik@kozaczko.info',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/django-getpaid/django-getpaid',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.6,<4.0',
}


setup(**setup_kwargs)



# Generated by Django 3.0.4 on 2020-04-03 15:54

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("getpaid", "0001_initial"),
    ]

    operations = [
        migrations.RenameField(
            model_name="payment", old_name="paid_on", new_name="last_payment_on",
        ),
        migrations.RemoveField(model_name="payment", name="amount",),
        migrations.AddField(
            model_name="payment",
            name="amount_locked",
            field=models.DecimalField(
                decimal_places=2, default=0, max_digits=20, verbose_name="amount paid"
            ),
        ),
        migrations.AddField(
            model_name="payment",
            name="amount_required",
            field=models.DecimalField(
                decimal_places=2,
                default=0,
                help_text="Amount in selected currency, normal notation",
                max_digits=20,
                verbose_name="amount required",
            ),
            preserve_default=False,
        ),
        migrations.AlterField(
            model_name="payment",
            name="amount_paid",
            field=models.DecimalField(
                decimal_places=2, default=0, max_digits=20, verbose_name="amount paid"
            ),
        ),
        migrations.AlterField(
            model_name="payment",
            name="backend",
            field=models.CharField(
                db_index=True, max_length=100, verbose_name="backend"
            ),
        ),
    ]



import swapper
from django import forms
from django.utils.translation import gettext_lazy as _

from getpaid.validators import run_getpaid_validators

from .registry import registry

Order = swapper.load_model("getpaid", "Order")


class PaymentMethodForm(forms.ModelForm):
    """
    Displays all available payments backends as choice list.
    """

    order = forms.ModelChoiceField(
        widget=forms.HiddenInput, queryset=Order.objects.all()
    )

    class Meta:
        model = swapper.load_model("getpaid", "Payment")
        fields = ["order", "amount_required", "description", "currency", "backend"]
        widgets = {
            "amount_required": forms.HiddenInput,
            "description": forms.HiddenInput,
            "currency": forms.HiddenInput,
        }

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        order = self.initial.get("order")
        currency = getattr(order, "currency", None) or self.data.get("currency")
        if order is not None:
            self.initial["amount_required"] = order.get_total_amount()
            self.initial["description"] = order.get_description()
        backends = registry.get_choices(currency)
        self.fields["backend"] = forms.ChoiceField(
            choices=backends,
            initial=backends[0][0] if len(backends) == 1 else "",
            label=_("Payment method"),
            widget=forms.RadioSelect,
        )

    def clean_order(self):
        if hasattr(self.cleaned_data["order"], "is_ready_for_payment"):
            if not self.cleaned_data["order"].is_ready_for_payment():
                raise forms.ValidationError(_("Order cannot be paid"))
        return self.cleaned_data["order"]

    def clean(self):
        cleaned_data = super().clean()
        return run_getpaid_validators(cleaned_data)


class PaymentHiddenInputsPostForm(forms.Form):
    def __init__(self, items, *args, **kwargs):
        super(PaymentHiddenInputsPostForm, self).__init__(*args, **kwargs)

        for key in items:
            self.fields[key] = forms.CharField(
                initial=items[key], widget=forms.HiddenInput
            )
